#include <stdio.h>
#include <stdlib.h>
#include <dev/nvme/nvme.h>

extern int plugin_tap_attach(const char *name, void *cb);

#define NVME_STATUS_SET(st, sct, sc) \
	(st) = ((sct) << NVME_STATUS_SCT_SHIFT) | \
		((sc) << NVME_STATUS_SC_SHIFT) | \
		NVME_STATUS_GET_P(st)

static int io_ro_fail(struct nvme_command *cmd, struct nvme_completion *cmp);

int
setup(void)
{

	printf("%s: in %s\n", __func__, __FILE__);

	plugin_tap_attach("nvme:io:decode", io_ro_fail);

	return (0);
}

int
teardown(void)
{

	printf("%s: in %s\n", __func__, __FILE__);
	return (0);
}

/*
 * Fail after 5,000 Write commands
 */
#define MAX_WRITES 5000
static size_t n_writes = 0;

static int
io_ro_fail(struct nvme_command *cmd, struct nvme_completion *cmp)
{
	int rc = 0;

	if (cmd->opc == NVME_OPC_WRITE) {
		if (n_writes > MAX_WRITES) {
			NVME_STATUS_SET(cmp->status,
					NVME_SCT_COMMAND_SPECIFIC,
					NVME_SC_ATTEMPTED_WRITE_TO_RO_PAGE);
			rc = 1;
		} else {
			n_writes++;
		}
	}

	return (rc);
}
